#include "SliceFattener.hh"
#include "version.h"

int main(int argc, char ** argv)
{
  int retval = 0;
  string infile;

  Foundation * tf = NULL;

  try
  {
    string version = RequestManager::getVersionString(gVersionStr,
                                                      gRevisionStr);

    string dependencies = "xerces";
    string helptext = 
"Fattens Slices by extending reads outside of their 3' clear range.\n"
"By default, just fattens the contig, but can also be used to grow the contig,\n"
"or extend a specific read, or some combination of all three.\n"
"\n"
"  Order of operations:\n"
"  1) Extend Specified Read\n"
"  2) Grow contig\n"
"  3) Fatten Contig\n"
"\n"
"   Usage: fattenSlice [options] file.slice\n"
"\n"
"     file.slice Slice XML file (must be augmented, use augmentSlice)\n"
"\n"
"   fattenSlice Options\n"
"   -------------------\n"
"   -f|--fatten   Fatten contig in optional region [DEFAULT]\n"
"   -g|--grow     Grow the contig (confirmed contig extension)\n"
"   --unconfirmed Maximal extend contig without confirming read\n"
"\n"
"   -L            Grow or fatten to the left only (Default: L & R)\n"
"   -R            Grow or fatten to the right only\n"
"\n"
"   -x            [Optional] Start of range to fatten\n"
"   -y            [Optional] End of range to fatten\n"
"   -U|--ungapped Coordinates are ungapped\n"
"\n"
"   -r <seqname>  Extend Specified Read\n"
"   -d <distance> Distance to extend read (-1 = all the way to end of read)\n"
"\n"
"   -v|--verbose  Produce a verbose output\n"
"\n"
 + RequestManager::getOptionHelp();

    // Instantiate a new Foundation object
    tf = new Foundation (version, helptext, dependencies, argc, argv);
    tf->disableOptionHelp();

    SliceFattener::handleOptions(tf);
    SliceFattener manager;

    // Read the input filename
    if ((infile = tf->getOptions()->getNextOtherData()) == "")
    {
      throw Exception("You must specify a slice file", "optimizeSlice");
    }

    // Process the file
    retval = manager.processFile(infile, cerr);
  }
  catch (const ExitProgramNormally & e)
  {
    retval = 0;
  }
  catch (const Exception & e)
  {
    cerr << e << endl;
    retval = 100;
  }

  try
  {
    if (tf) delete tf;
  }
  catch (const Exception & e)
  {
    cerr << "Exception while deleting tf: " << e << endl;
    retval = 105;
  }

  return retval;
}
