#include "TilingManager.hh"

TilingManager::TilingManager() : RequestManager("TilingManager")
{
  m_contigCount = 0;
  m_consensusSequence = NULL;
}

string TilingManager::m_asmblId("");
int TilingManager::m_circular(0);


TilingManager::~TilingManager()
{
  if (m_consensusSequence) { delete m_consensusSequence; }
}

void TilingManager::addSliceRange(long & index, long & gindex)
{
  if (m_consensusSequence)
  {
    m_consensusSequence->printContig(m_asmblId);
    delete m_consensusSequence;
  }

  m_consensusSequence = new ConsensusSequence(m_contigCount, 
                                              index, 
                                              m_out, 
                                              m_circular);
  
  m_contigCount++;

  index = gindex = 0L;
}

void TilingManager::addSlice(SliceData * slice)
{
  RequestManager::addSlice(slice);
  m_consensusSequence->addSlice(slice);
  delete slice;
}


int TilingManager::printResults()
{
  if (!m_errorCount && m_consensusSequence)
  {
    m_consensusSequence->printContig(m_asmblId);
  }

  return m_errorCount;
}

void TilingManager::setAsmblId(const string & asmblId)
{
  m_asmblId = asmblId;
}

void TilingManager::handleOptions(Foundation * tf)
{
  tf->getOptions()->addOptionResult("asmbl_id|i=s", &m_asmblId,
                                    "Specify the assembly id");

  tf->getOptions()->addOptionResult("C", &m_circular,
                                    "Allow for circular contig (negative offsets on reads)");
  RequestManager::handleOptions(tf);

  if (m_asmblId.empty()) { m_asmblId = "0"; }
}
