#include "SequenceData.hh"
#include "Foundation.hh"
#include "Slice.h"

SequenceData::SequenceData(const ReadData * rd, long offset, long index)
{
  if (!rd)
  {
    throw Exception("ReadData cannot be NULL",
                        "SequenceData::SequenceData");
  }

  m_readData = rd;
  m_offset = offset;

  m_asmleft = m_asmright = index;

  m_reverseCompliment = m_readData->isReverseCompliment();

  m_nuc = m_qualVal = "";
  m_gapCount = 0;
}

SequenceData::~SequenceData()
{ }

void SequenceData::addElement(long index, char nuc, char qualval)
{
  if (nuc == '-')
  {
    m_gapList.push_back(m_nuc.length() + 1);
    m_gapCount++;
  }
  else
  {
    m_asmright = index;
    m_nuc += nuc;
    m_qualVal += qualval;
  }
}

void SequenceData::printNucXML(ostream & out, const string & prefix) const
{
  out << prefix << "<Nuc>";

  out << m_readData->getNucData(true);

  if (m_reverseCompliment)
  {
    string::const_iterator j;
    for (j =  m_nuc.end()-1;
         j != m_nuc.begin()-1;
         j--)
    {
      out << libSlice_getCompliment(*j);
    }
  }
  else
  {
    out << m_nuc;
  }

  out << m_readData->getNucData(false);
  
  out << "</Nuc>" << endl;
}

void SequenceData::printQualValXML(ostream & out, const string & prefix) const
{
  out << prefix << "<Qualval>";

  bool first = true;
  string::const_iterator j;

  const string & leftTrimmed  = m_readData->getQualData(true);
  const string & rightTrimmed = m_readData->getQualData(false);

  if (leftTrimmed.length())
  {
    out << leftTrimmed;
    first = false;
  }

  if (m_reverseCompliment)
  {
    for (j =  m_qualVal.end()-1;
         j != m_qualVal.begin()-1;
         j--)
    {
      if (!first) out << ' ';
      out << (int) *j;
      first = false;
    }
  }
  else
  {
    for (j =  m_qualVal.begin();
         j != m_qualVal.end();
         j++)
    {
      if (!first) out << ' ';
      out << (int) *j;
      first = false;
    }
  }

  if (rightTrimmed.length())
  {
    if (!first) out << ' ';
    out << rightTrimmed;
  }

  out << "</Qualval>" << endl;
}

const string & SequenceData::getId() const
{
  return m_readData->getId();
}

void SequenceData::printSequenceXML(ostream & out, const string & prefix)
{
  out << prefix 
      << "<Sequence Id=\"" << m_readData->getId()
      << "\" Name=\""      << m_readData->getSeqname()
      << "\">" << endl;

  printNucXML    (out, prefix+"  ");
  printQualValXML(out, prefix+"  ");

  out << prefix 
      << "  <Clr Left=\"" << m_readData->getClrleft()
      << "\" Right=\""    << m_readData->getClrright()
      << "\" />" << endl;

  out << prefix << "</Sequence>" << endl;

  m_nuc = "";
  m_qualVal = "";
}


void SequenceData::printSeqRecordXML(ostream & out, const string & prefix,
                                     int coordinateOffset)
{
  out << prefix << "<Seq Id=\"" << m_readData->getId() << "\">" << endl;

  if (m_gapCount)
  {
    out << prefix << "  <Gaps>";

    bool first = true;
    GapList::const_iterator i;
    for (i =  m_gapList.begin();
         i != m_gapList.end();
         i++)
    {
      if (!first) out << ' ';
      out << (*i);
      first = false;
    }

    out << "</Gaps>" << endl;
  }
  else
  {
    out << prefix << "  <Gaps />" << endl;
  }

  out << prefix 
      << "  <Seqrange Left=\"" << m_readData->getSeqleft()
      << "\" Right=\""         << m_readData->getSeqright()
      << "\" />" << endl;

  out << prefix 
      << "  <Asmrange Left=\"" << m_asmleft + coordinateOffset
      << "\" Right=\""         << m_asmright + coordinateOffset
      << "\" />" << endl;

  out << prefix << "  <Offset>" << m_offset << "</Offset>" << endl
      << prefix << "</Seq>" << endl;
}
