#include "SliceZipper.hh"
#include "version.h"

int main(int argc, char ** argv)
{
  int retval = 0;
  Foundation * tf = NULL;

  try
  {
    string version = RequestManager::getVersionString(gVersionStr,
                                                      gRevisionStr);
    string dependencies = "xerces";
    string helptext = 
"Zip together a query slice file onto a reference slice file. The beginning\n"
"of the alignment is specified with -O, and gaps can be inserted into both\n"
"slice files using -r and -q.\n"
"\n"
"   Usage: zipSlice [options] query.slice reference.slice\n"
"\n"
"   query.slice     Slice file of query contig (typically smaller than reference)\n"
"   reference.slice Slice file of reference contig\n"
"\n"
"   zipSlice Options\n"
"   ----------------\n"
"   -R Query should be reversed before merging\n"
"   -1 Coordinates of gaps are 1-based\n"
"   -U Coordinates of gaps to insert are ungapped\n"
"   -p Automatically promote preexisting gaps\n"
"   --refcons Use the consensus of the reference for merged slices\n"
"             (Uses the query consensus by default)\n"
"\n"
"   -q <csl> Comma separated list of positions to insert gaps in the query\n"
"   -r <csl> Comma separated list of positions to insert gaps in the reference\n"
"\n"
"   -O <offset> Specify alignment offset from query to reference\n"
"               (negative means reference left flanks query (always 0-based))\n"
 + RequestManager::getOptionHelp();


    // Instantiate a new Foundation object
    tf = new Foundation (version, helptext, dependencies, argc, argv);
    tf->disableOptionHelp();

    SliceZipper::handleOptions(tf);
    
    list<string> inputfiles = tf->getOptions()->getAllOtherData();
    list<string>::const_iterator infile;

    if (inputfiles.size() != 2)
    {
      throw Exception("You must specify two files to zipper",
                          "zipSlice");
    }

    SliceZipper sliceZipper;

    // Pass the indel list here

    // Do first pass to load read data, sort offsets
    for (infile =  inputfiles.begin();
         infile != inputfiles.end();
         infile++)
    {
      retval += sliceZipper.processFile(*infile, cerr);
    }

    retval += sliceZipper.endSlices();
  }
  catch (const ExitProgramNormally & e)
  {
    retval = 0;
  }
  catch (const Exception & e)
  {
    cerr << e << endl;
    retval = 100;
  }

  try
  {
    if (tf) delete tf;
  }
  catch (const Exception & e)
  {
    cerr << "Exception while deleting tf: " << e << endl;
    retval = 105;
  }

  return retval;
}
